@@ -218,28 +218,6 @@ atanh (const Complex& x)
   return log ((1.0 + x) / (1.0 - x)) / 2.0;
 }
 
-#if !defined (CXX_ISO_COMPLIANT_LIBRARY)
-
-Complex
-log10 (const Complex& x)
-{
-  return M_LOG10E * log (x);
-}
-
-Complex
-tan (const Complex& x)
-{
-  return sin (x) / cos (x);
-}
-
-Complex
-tanh (const Complex& x)
-{
-  return sinh (x) / cosh (x);
-}
-
-#endif
-
 Complex
 ceil (const Complex& x)
 {
