@@ -21,10 +21,6 @@ Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 
 */
 
-#if defined (__GNUG__) && defined (USE_PRAGMA_INTERFACE_IMPLEMENTATION)
-#pragma implementation
-#endif
-
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -43,6 +39,9 @@ Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 #include "ov-cell.h"
 #include "ov-scalar.h"
 #include "ov-re-mat.h"
+#include "ov-bool-sparse.h"
+#include "ov-cx-sparse.h"
+#include "ov-re-sparse.h"
 #include "ov-int8.h"
 #include "ov-int16.h"
 #include "ov-int32.h"
@@ -57,7 +56,6 @@ Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 #include "ov-str-mat.h"
 #include "ov-range.h"
 #include "ov-struct.h"
-#include "ov-file.h"
 #include "ov-streamoff.h"
 #include "ov-list.h"
 #include "ov-cs-list.h"
@@ -552,61 +550,96 @@ octave_value::octave_value (const boolNDArray& bnda)
   maybe_mutate ();
 }
 
-octave_value::octave_value (char c)
-  : rep (new octave_char_matrix_str (c))
+octave_value::octave_value (char c, char type)
+  : rep (type == '"'
+	 ? new octave_char_matrix_dq_str (c)
+	 : new octave_char_matrix_sq_str (c))
 {
   rep->count = 1;
   maybe_mutate ();
 }
 
-octave_value::octave_value (const char *s)
-  : rep (new octave_char_matrix_str (s))
+octave_value::octave_value (const char *s, char type)
+  : rep (type == '"'
+	 ? new octave_char_matrix_dq_str (s)
+	 : new octave_char_matrix_sq_str (s))
 {
   rep->count = 1;
   maybe_mutate ();
 }
 
-octave_value::octave_value (const std::string& s)
-  : rep (new octave_char_matrix_str (s))
+octave_value::octave_value (const std::string& s, char type)
+  : rep (type == '"'
+	 ? new octave_char_matrix_dq_str (s)
+	 : new octave_char_matrix_sq_str (s))
 {
   rep->count = 1;
   maybe_mutate ();
 }
 
-octave_value::octave_value (const string_vector& s)
-  : rep (new octave_char_matrix_str (s))
+octave_value::octave_value (const string_vector& s, char type)
+  : rep (type == '"'
+	 ? new octave_char_matrix_dq_str (s)
+	 : new octave_char_matrix_sq_str (s))
 {
   rep->count = 1;
   maybe_mutate ();
 }
 
-octave_value::octave_value (const charMatrix& chm, bool is_str)
+octave_value::octave_value (const charMatrix& chm, bool is_str, char type)
   : rep (is_str
-	 ? new octave_char_matrix_str (chm)
+	 ? (type == '"'
+	    ? new octave_char_matrix_dq_str (chm)
+	    : new octave_char_matrix_sq_str (chm))
 	 : new octave_char_matrix (chm))
 {
   rep->count = 1;
   maybe_mutate ();
 }
 
-octave_value::octave_value (const charNDArray& chm, bool is_str)
+octave_value::octave_value (const charNDArray& chm, bool is_str, char type)
   : rep (is_str
-	 ? new octave_char_matrix_str (chm)
+	 ? (type == '"'
+	    ? new octave_char_matrix_dq_str (chm)
+	    : new octave_char_matrix_sq_str (chm))
 	 : new octave_char_matrix (chm))
 {
   rep->count = 1;
   maybe_mutate ();
 }
 
-octave_value::octave_value (const ArrayN<char>& chm, bool is_str)
+octave_value::octave_value (const ArrayN<char>& chm, bool is_str, char type)
   : rep (is_str
-	 ? new octave_char_matrix_str (chm)
+	 ? (type == '"'
+	    ? new octave_char_matrix_dq_str (chm)
+	    : new octave_char_matrix_sq_str (chm))
 	 : new octave_char_matrix (chm))
 {
   rep->count = 1;
   maybe_mutate ();
 }
 
+octave_value::octave_value (const SparseMatrix& m, const SparseType &t)
+  : rep (new octave_sparse_matrix (m, t))
+{
+  rep->count = 1;
+  maybe_mutate ();
+}
+
+octave_value::octave_value (const SparseComplexMatrix& m, const SparseType &t)
+  : rep (new octave_sparse_complex_matrix (m, t))
+{
+  rep->count = 1;
+  maybe_mutate ();
+}
+
+octave_value::octave_value (const SparseBoolMatrix& bm, const SparseType &t)
+  : rep (new octave_sparse_bool_matrix (bm, t))
+{
+  rep->count = 1;
+  maybe_mutate ();
+}
+
 octave_value::octave_value (const octave_int8& i)
   : rep (new octave_int8_scalar (i))
 {
@@ -739,12 +772,6 @@ octave_value::octave_value (const Octave_map& m)
   rep->count = 1;
 }
 
-octave_value::octave_value (const octave_stream& s, int n)
-  : rep (new octave_file (s, n))
-{
-  rep->count = 1;
-}
-
 octave_value::octave_value (const streamoff_array& off)
   : rep (new octave_streamoff (off))
 {
@@ -993,7 +1020,7 @@ octave_value::assign (assign_op op, const octave_value& rhs)
   return *this;
 }
 
-int
+octave_idx_type
 octave_value::rows (void) const
 {
   dim_vector dv = dims ();
@@ -1001,7 +1028,7 @@ octave_value::rows (void) const
   return (dv.length () > 0) ? dv(0) : -1;
 }
 
-int
+octave_idx_type
 octave_value::columns (void) const
 {
   dim_vector dv = dims ();
@@ -1009,7 +1036,7 @@ octave_value::columns (void) const
   return (dv.length () > 1) ? dv(1) : -1;
 }
 
-int
+octave_idx_type
 octave_value::length (void) const
 {
   int retval = 0;
@@ -1074,18 +1101,6 @@ octave_value::map_value (void) const
   return rep->map_value ();
 }
 
-octave_stream
-octave_value::stream_value (void) const
-{
-  return rep->stream_value ();
-}
-
-int
-octave_value::stream_number (void) const
-{
-  return rep->stream_number ();
-}
-
 std::streamoff
 octave_value::streamoff_value (void) const
 {
@@ -1139,13 +1154,13 @@ octave_value::column_vector_value (bool force_string_conv,
   if (error_state)
     return retval;
 
-  int nr = m.rows ();
-  int nc = m.columns ();
+  octave_idx_type nr = m.rows ();
+  octave_idx_type nc = m.columns ();
 
   if (nc == 1)
     {
       retval.resize (nr);
-      for (int i = 0; i < nr; i++)
+      for (octave_idx_type i = 0; i < nr; i++)
 	retval (i) = m (i, 0);
     }
   else
@@ -1168,13 +1183,13 @@ octave_value::complex_column_vector_value (bool force_string_conv,
   if (error_state)
     return retval;
 
-  int nr = m.rows ();
-  int nc = m.columns ();
+  octave_idx_type nr = m.rows ();
+  octave_idx_type nc = m.columns ();
 
   if (nc == 1)
     {
       retval.resize (nr);
-      for (int i = 0; i < nr; i++)
+      for (octave_idx_type i = 0; i < nr; i++)
 	retval (i) = m (i, 0);
     }
   else
@@ -1197,13 +1212,13 @@ octave_value::row_vector_value (bool force_string_conv,
   if (error_state)
     return retval;
 
-  int nr = m.rows ();
-  int nc = m.columns ();
+  octave_idx_type nr = m.rows ();
+  octave_idx_type nc = m.columns ();
 
   if (nr == 1)
     {
       retval.resize (nc);
-      for (int i = 0; i < nc; i++)
+      for (octave_idx_type i = 0; i < nc; i++)
 	retval (i) = m (0, i);
     }
   else
@@ -1226,13 +1241,13 @@ octave_value::complex_row_vector_value (bool force_string_conv,
   if (error_state)
     return retval;
 
-  int nr = m.rows ();
-  int nc = m.columns ();
+  octave_idx_type nr = m.rows ();
+  octave_idx_type nc = m.columns ();
 
   if (nr == 1)
     {
       retval.resize (nc);
-      for (int i = 0; i < nc; i++)
+      for (octave_idx_type i = 0; i < nc; i++)
 	retval (i) = m (0, i);
     }
   else
@@ -1257,19 +1272,19 @@ octave_value::vector_value (bool force_string_conv,
   if (error_state)
     return retval;
 
-  int nr = m.rows ();
-  int nc = m.columns ();
+  octave_idx_type nr = m.rows ();
+  octave_idx_type nc = m.columns ();
 
   if (nr == 1)
     {
       retval.resize (nc);
-      for (int i = 0; i < nc; i++)
+      for (octave_idx_type i = 0; i < nc; i++)
 	retval (i) = m (0, i);
     }
   else if (nc == 1)
     {
       retval.resize (nr);
-      for (int i = 0; i < nr; i++)
+      for (octave_idx_type i = 0; i < nr; i++)
 	retval (i) = m (i, 0);
     }
   else if (nr > 0 && nc > 0)
@@ -1279,9 +1294,9 @@ octave_value::vector_value (bool force_string_conv,
 	gripe_implicit_conversion (type_name (), "real vector");
 
       retval.resize (nr * nc);
-      int k = 0;
-      for (int j = 0; j < nc; j++)
-	for (int i = 0; i < nr; i++)
+      octave_idx_type k = 0;
+      for (octave_idx_type j = 0; j < nc; j++)
+	for (octave_idx_type i = 0; i < nr; i++)
 	  {
 	    OCTAVE_QUIT;
 
@@ -1308,13 +1323,13 @@ octave_value::int_vector_value (bool force_string_conv, bool require_int,
   if (error_state)
     return retval;
 
-  int nr = m.rows ();
-  int nc = m.columns ();
+  octave_idx_type nr = m.rows ();
+  octave_idx_type nc = m.columns ();
 
   if (nr == 1)
     {
       retval.resize (nc);
-      for (int i = 0; i < nc; i++)
+      for (octave_idx_type i = 0; i < nc; i++)
 	{
 	  OCTAVE_QUIT;
 
@@ -1332,7 +1347,7 @@ octave_value::int_vector_value (bool force_string_conv, bool require_int,
   else if (nc == 1)
     {
       retval.resize (nr);
-      for (int i = 0; i < nr; i++)
+      for (octave_idx_type i = 0; i < nr; i++)
 	{
 	  OCTAVE_QUIT;
 
@@ -1354,10 +1369,10 @@ octave_value::int_vector_value (bool force_string_conv, bool require_int,
 	gripe_implicit_conversion (type_name (), "real vector");
 
       retval.resize (nr * nc);
-      int k = 0;
-      for (int j = 0; j < nc; j++)
+      octave_idx_type k = 0;
+      for (octave_idx_type j = 0; j < nc; j++)
 	{
-	  for (int i = 0; i < nr; i++)
+	  for (octave_idx_type i = 0; i < nr; i++)
 	    {
 	      OCTAVE_QUIT;
 
@@ -1393,13 +1408,13 @@ octave_value::complex_vector_value (bool force_string_conv,
   if (error_state)
     return retval;
 
-  int nr = m.rows ();
-  int nc = m.columns ();
+  octave_idx_type nr = m.rows ();
+  octave_idx_type nc = m.columns ();
 
   if (nr == 1)
     {
       retval.resize (nc);
-      for (int i = 0; i < nc; i++)
+      for (octave_idx_type i = 0; i < nc; i++)
 	{
 	  OCTAVE_QUIT;
 	  retval (i) = m (0, i);
@@ -1408,7 +1423,7 @@ octave_value::complex_vector_value (bool force_string_conv,
   else if (nc == 1)
     {
       retval.resize (nr);
-      for (int i = 0; i < nr; i++)
+      for (octave_idx_type i = 0; i < nr; i++)
 	{
 	  OCTAVE_QUIT;
 	  retval (i) = m (i, 0);
@@ -1421,9 +1436,9 @@ octave_value::complex_vector_value (bool force_string_conv,
 	gripe_implicit_conversion (type_name (), "complex vector");
 
       retval.resize (nr * nc);
-      int k = 0;
-      for (int j = 0; j < nc; j++)
-	for (int i = 0; i < nr; i++)
+      octave_idx_type k = 0;
+      for (octave_idx_type j = 0; j < nc; j++)
+	for (octave_idx_type i = 0; i < nr; i++)
 	  {
 	    OCTAVE_QUIT;
 	    retval (k++) = m (i, j);
@@ -1439,9 +1454,9 @@ octave_value::complex_vector_value (bool force_string_conv,
 }
 
 octave_value
-octave_value::convert_to_str (bool pad, bool force) const
+octave_value::convert_to_str (bool pad, bool force, char type) const
 {
-  octave_value retval = convert_to_str_internal (pad, force);
+  octave_value retval = convert_to_str_internal (pad, force, type);
 
   if (! force && is_numeric_type () && Vwarn_num_to_str)
     gripe_implicit_conversion (type_name (), retval.type_name ());
@@ -1710,6 +1725,13 @@ do_cat_op (const octave_value& v1, const octave_value& v2,
 {
   octave_value retval;
 
+  // Rapid return for concatenation with an empty object. Dimension
+  // checking handled elsewhere.
+  if (v1.all_zero_dims ())
+    return v2;
+  if (v2.all_zero_dims ())
+    return v1;
+
   int t1 = v1.type_id ();
   int t2 = v2.type_id ();
 
@@ -2154,6 +2176,7 @@ install_types (void)
   octave_bool_matrix::register_type ();
   octave_char_matrix::register_type ();
   octave_char_matrix_str::register_type ();
+  octave_char_matrix_sq_str::register_type ();
   octave_int8_scalar::register_type ();
   octave_int16_scalar::register_type ();
   octave_int32_scalar::register_type ();
@@ -2170,8 +2193,10 @@ install_types (void)
   octave_uint16_matrix::register_type ();
   octave_uint32_matrix::register_type ();
   octave_uint64_matrix::register_type ();
+  octave_sparse_bool_matrix::register_type ();
+  octave_sparse_matrix::register_type ();
+  octave_sparse_complex_matrix::register_type ();
   octave_struct::register_type ();
-  octave_file::register_type ();
   octave_list::register_type ();
   octave_cs_list::register_type ();
   octave_all_va_args::register_type ();
