@@ -24,10 +24,6 @@ Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 #if !defined (octave_value_h)
 #define octave_value_h 1
 
-#if defined (__GNUG__) && defined (USE_PRAGMA_INTERFACE_IMPLEMENTATION)
-#pragma interface
-#endif
-
 #include <cstdlib>
 
 #include <iostream>
@@ -44,6 +40,7 @@ Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 #include "oct-alloc.h"
 #include "oct-time.h"
 #include "str-vec.h"
+#include "SparseType.h"
 
 class Cell;
 class streamoff_array;
@@ -212,13 +209,21 @@ public:
   octave_value (bool b);
   octave_value (const boolMatrix& bm);
   octave_value (const boolNDArray& bnda);
-  octave_value (char c);
-  octave_value (const char *s);
-  octave_value (const std::string& s);
-  octave_value (const string_vector& s);
-  octave_value (const charMatrix& chm, bool is_string = false);
-  octave_value (const charNDArray& chnda, bool is_string = false);
-  octave_value (const ArrayN<char>& chnda, bool is_string = false);
+  octave_value (char c, char type = '"');
+  octave_value (const char *s, char type = '"');
+  octave_value (const std::string& s, char type = '"');
+  octave_value (const string_vector& s, char type = '"');
+  octave_value (const charMatrix& chm, bool is_string = false,
+		char type = '"');
+  octave_value (const charNDArray& chnda, bool is_string = false,
+		char type = '"');
+  octave_value (const ArrayN<char>& chnda, bool is_string = false,
+		char type = '"');
+  octave_value (const SparseMatrix& m, const SparseType& t = SparseType ());
+  octave_value (const SparseComplexMatrix& m, 
+		const SparseType& t = SparseType ());
+  octave_value (const SparseBoolMatrix& bm, 
+		const SparseType& t = SparseType ());
   octave_value (const octave_int8& i);
   octave_value (const octave_int16& i);
   octave_value (const octave_int32& i);
@@ -238,7 +243,6 @@ public:
   octave_value (double base, double limit, double inc);
   octave_value (const Range& r);
   octave_value (const Octave_map& m);
-  octave_value (const octave_stream& s, int n);
   octave_value (const streamoff_array& off);
   octave_value (const octave_value_list& m, bool is_cs_list = false);
   octave_value (octave_value::magic_colon);
@@ -353,17 +357,22 @@ public:
   virtual dim_vector dims (void) const
     { return rep->dims (); }
 
-  int rows (void) const;
+  octave_idx_type rows (void) const;
 
-  int columns (void) const;
+  octave_idx_type columns (void) const;
 
-  int length (void) const;
+  octave_idx_type length (void) const;
 
   int ndims (void) const;
 
-  virtual int numel (void) const
+  bool all_zero_dims (void) const { return dims().all_zero (); }
+
+  virtual octave_idx_type numel (void) const
     { return rep->numel (); }
 
+  virtual octave_idx_type capacity (void) const
+    { return rep->capacity (); }
+
   virtual size_t byte_size (void) const
     { return rep->byte_size (); }
 
@@ -419,15 +428,18 @@ public:
   virtual bool is_string (void) const
     { return rep->is_string (); }
 
+  virtual bool is_sq_string (void) const
+    { return rep->is_sq_string (); }
+
+  bool is_dq_string (void) const
+    { return rep->is_string () && ! rep->is_sq_string (); }
+
   virtual bool is_range (void) const
     { return rep->is_range (); }
 
   virtual bool is_map (void) const
     { return rep->is_map (); }
 
-  virtual bool is_stream (void) const
-    { return rep->is_stream (); }
-
   virtual bool is_streamoff (void) const
     { return rep->is_streamoff (); }
 
@@ -573,6 +585,15 @@ public:
   virtual charNDArray char_array_value (bool frc_str_conv = false) const
     { return rep->char_array_value (frc_str_conv); }
 
+  virtual SparseMatrix sparse_matrix_value (bool frc_str_conv = false) const
+  { return rep->sparse_matrix_value (frc_str_conv); }
+
+  virtual SparseComplexMatrix sparse_complex_matrix_value (bool frc_str_conv = false) const
+  { return rep->sparse_complex_matrix_value (frc_str_conv); }
+
+  virtual SparseBoolMatrix sparse_bool_matrix_value (bool frc_str_conv = false) const
+  { return rep->sparse_bool_matrix_value (frc_str_conv); }
+
   virtual octave_int8 int8_scalar_value (void) const
     { return rep->int8_scalar_value (); }
 
@@ -636,10 +657,6 @@ public:
   virtual string_vector map_keys (void) const
     { return rep->map_keys (); }
 
-  virtual octave_stream stream_value (void) const;
-
-  virtual int stream_number (void) const;
-
   virtual std::streamoff streamoff_value (void) const;
 
   virtual streamoff_array streamoff_array_value (void) const;
@@ -682,10 +699,12 @@ public:
   // class wants a certain kind of constant, he should simply ask for
   // it, and we should convert it if possible.
 
-  octave_value convert_to_str (bool pad = false, bool force = false) const;
+  octave_value convert_to_str (bool pad = false, bool force = false,
+			       char type = '"') const;
 
-  virtual octave_value convert_to_str_internal (bool pad, bool force) const
-    { return rep->convert_to_str_internal (pad, force); }
+  virtual octave_value
+  convert_to_str_internal (bool pad, bool force, char type) const
+    { return rep->convert_to_str_internal (pad, force, type); }
 
   virtual void convert_to_row_or_column_vector (void)
     { rep->convert_to_row_or_column_vector (); }
@@ -814,15 +833,6 @@ private:
   octave_value *nil_rep (void) const;
 };
 
-// Publish externally used friend functions.
-
-extern octave_value
-do_unary_op (octave_value::unary_op op, const octave_value& a);
-
-extern octave_value
-do_binary_op (octave_value::binary_op op,
-	      const octave_value& a, const octave_value& b);
-
 #define OV_UNOP_FN(name) \
   inline octave_value \
   name (const octave_value& a) \
